"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractEventBus = void 0;
// event bus
const events_1 = require("events");
const interface_1 = require("./interface");
const util_1 = require("util");
const error_1 = require("./error");
const DEFAULT_LISTENER_KEY = '_default_';
class AbstractEventBus {
    constructor(options = {}) {
        this.options = options;
        this.isInited = false;
        this.workers = [];
        this.stopping = false;
        this.hub = new events_1.EventEmitter();
        this.workerReady = new Map();
        this.topicListener = new Map();
        this.asyncMessageMap = new Map();
        this.eventListenerMap = new Map();
        this.debugLogger = this.createDebugger();
        this.debugLogger(`Start EventBus(${this.constructor.name}) in ${this.isWorker() ? 'worker' : 'main'}`);
        this.eventListenerMap.set(interface_1.ListenerType.Error, (err) => {
            console.error(err);
        });
        this.listener = (message, callback) => {
            var _a;
            const listeners = this.topicListener.get(((_a = message.messageOptions) === null || _a === void 0 ? void 0 : _a.topic) || DEFAULT_LISTENER_KEY);
            if (listeners) {
                for (const listener of listeners) {
                    if (listener['_subscribeOnce']) {
                        listeners.delete(listener);
                    }
                    listener(message, callback);
                }
            }
        };
        // bind event center
        this.setupEventBind();
    }
    createDebugger() {
        return (0, util_1.debuglog)(`midway:event-bus:${this.isWorker() ? 'worker' : 'main  '}`);
    }
    debugDataflow(message) {
        if (message.messageCategory === interface_1.MessageCategory.IN) {
            if (this.isMain()) {
                return `${message.message.type}|${message.messageCategory}: worker => main(△)`;
            }
            else {
                return `${message.message.type}|${message.messageCategory}: main => worker(△)`;
            }
        }
        else {
            if (this.isMain()) {
                return `${message.message.type}|${message.messageCategory}: main(△) => worker`;
            }
            else {
                return `${message.message.type}|${message.messageCategory}: worker(△) => main`;
            }
        }
    }
    async start() {
        this.isInited = true;
        if (this.isMain()) {
            await new Promise((resolve, reject) => {
                const timeoutHandler = setTimeout(() => {
                    clearInterval(handler);
                    clearTimeout(timeoutHandler);
                    reject(new error_1.EventBusWaitWorkerInitedTimeoutError());
                }, this.options.initTimeout || 5000);
                const handler = setInterval(() => {
                    if (this.isAllWorkerReady()) {
                        clearInterval(handler);
                        clearTimeout(timeoutHandler);
                        resolve(true);
                    }
                }, 200);
            });
        }
        else {
            // listen main => worker in worker
            this.workerSubscribeMessage((message) => {
                this.transit({
                    messageCategory: interface_1.MessageCategory.IN,
                    message,
                });
            });
            // worker => main
            this.transit({
                messageCategory: interface_1.MessageCategory.OUT,
                message: {
                    messageId: this.generateMessageId(),
                    workerId: this.getWorkerId(),
                    type: interface_1.MessageType.Inited,
                    body: this.isInited,
                },
            });
        }
    }
    addWorker(worker) {
        this.debugLogger(`Add worker(${this.getWorkerId(worker)})`);
        if (!this.workerReady.has(this.getWorkerId(worker))) {
            this.debugLogger(`Init worker(${this.getWorkerId(worker)}) status = false`);
            this.workerReady.set(this.getWorkerId(worker), {
                worker,
                ready: false,
            });
        }
        else {
            this.debugLogger(`Skip init worker(${this.getWorkerId(worker)}) status`);
        }
        worker === null || worker === void 0 ? void 0 : worker['on']('exit', async (exitCode) => {
            if (!this.stopping) {
                // remove ready status
                this.workerReady.delete(this.getWorkerId(worker));
                // remove worker
                const idx = this.workers.findIndex(item => this.getWorkerId(item) === this.getWorkerId(worker));
                this.workers.splice(idx, 1);
            }
        });
        // listen worker => main in main
        this.workerListenMessage(worker, (message) => {
            this.transit({
                messageCategory: interface_1.MessageCategory.IN,
                message,
            });
        });
        this.workers.push(worker);
    }
    isAllWorkerReady() {
        for (const [workerId, value] of this.workerReady) {
            if (!value || !value.ready) {
                this.debugLogger(`Worker(${workerId}) not ready.`);
                return false;
            }
        }
        this.debugLogger(`All worker(size=${this.workerReady.size}) is ready.`);
        return true;
    }
    setupEventBind() {
        this.hub.on('message', (message) => {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
            if (!message.message || !message.message.messageId) {
                // ignore unvalid format message
                return;
            }
            this.debugLogger('EventCenter(%s) message = %j', this.debugDataflow(message), message);
            const originMessage = message.message;
            if (message.messageCategory === interface_1.MessageCategory.OUT) {
                // out operation
                if (originMessage.type === interface_1.MessageType.Invoke ||
                    originMessage.type === interface_1.MessageType.Request ||
                    originMessage.type === interface_1.MessageType.Response ||
                    originMessage.type === interface_1.MessageType.Broadcast) {
                    this.postMessage(originMessage);
                    (_a = this.eventListenerMap.get(interface_1.ListenerType.Request)) === null || _a === void 0 ? void 0 : _a(originMessage);
                }
                else if (originMessage.type === interface_1.MessageType.Inited) {
                    this.postMessage(originMessage);
                    (_b = this.eventListenerMap.get(interface_1.ListenerType.Inited)) === null || _b === void 0 ? void 0 : _b(originMessage);
                }
            }
            else if (message.messageCategory === interface_1.MessageCategory.IN) {
                // in operation
                if (originMessage.type === interface_1.MessageType.Invoke) {
                    (_c = this.listener) === null || _c === void 0 ? void 0 : _c.call(this, originMessage, data => {
                        this.publish(data, {
                            relatedMessageId: originMessage.messageId,
                        });
                    });
                    (_d = this.eventListenerMap.get(interface_1.ListenerType.Subscribe)) === null || _d === void 0 ? void 0 : _d(originMessage);
                }
                else if (originMessage.type === interface_1.MessageType.Request) {
                    (_e = this.listener) === null || _e === void 0 ? void 0 : _e.call(this, originMessage);
                    (_f = this.eventListenerMap.get(interface_1.ListenerType.Subscribe)) === null || _f === void 0 ? void 0 : _f(originMessage);
                }
                else if (originMessage.type === interface_1.MessageType.Broadcast) {
                    if (this.isMain()) {
                        if (originMessage.messageOptions['includeMainFromWorker'] === true) {
                            (_g = this.listener) === null || _g === void 0 ? void 0 : _g.call(this, originMessage);
                            (_h = this.eventListenerMap.get(interface_1.ListenerType.Subscribe)) === null || _h === void 0 ? void 0 : _h(originMessage);
                        }
                        this.broadcast(originMessage.body, {
                            ...originMessage.messageOptions,
                            relatedMessageId: originMessage.messageId,
                            relatedWorkerId: originMessage.workerId,
                        });
                    }
                    else {
                        (_j = this.listener) === null || _j === void 0 ? void 0 : _j.call(this, originMessage);
                        (_k = this.eventListenerMap.get(interface_1.ListenerType.Subscribe)) === null || _k === void 0 ? void 0 : _k(originMessage);
                    }
                }
                else if (originMessage.type === interface_1.MessageType.Response) {
                    if ((_l = originMessage.messageOptions) === null || _l === void 0 ? void 0 : _l.relatedMessageId) {
                        // worker => main with invoke
                        const asyncResolve = this.asyncMessageMap.get(originMessage.messageOptions.relatedMessageId);
                        if (asyncResolve) {
                            this.asyncMessageMap.delete(originMessage.messageOptions.relatedMessageId);
                            asyncResolve(originMessage.body);
                        }
                        else {
                            // not found and ignore
                        }
                    }
                    else {
                        (_m = this.listener) === null || _m === void 0 ? void 0 : _m.call(this, originMessage);
                        (_o = this.eventListenerMap.get(interface_1.ListenerType.Subscribe)) === null || _o === void 0 ? void 0 : _o(originMessage);
                    }
                }
                else if (originMessage.type === interface_1.MessageType.Inited) {
                    if (this.isMain()) {
                        // trigger in worker
                        (_p = this.eventListenerMap.get(interface_1.ListenerType.Inited)) === null || _p === void 0 ? void 0 : _p(originMessage);
                        // got init status from worker
                        this.workerReady.get(originMessage.workerId).ready = true;
                        this.debugLogger(`got worker ${originMessage.workerId} ready`);
                    }
                    else {
                        // ignore
                    }
                }
            }
        });
    }
    transit(message) {
        this.hub.emit('message', message);
    }
    subscribe(listener, options = {}) {
        if (!this.topicListener.has(options.topic)) {
            this.topicListener.set(options.topic || DEFAULT_LISTENER_KEY, new Set());
        }
        if (options.subscribeOnce) {
            listener['_subscribeOnce'] = true;
        }
        this.topicListener.get(options.topic || DEFAULT_LISTENER_KEY).add(listener);
    }
    subscribeOnce(listener, options = {}) {
        options.subscribeOnce = true;
        this.subscribe(listener, options);
    }
    publish(data, publishOptions = {}) {
        this.transit({
            messageCategory: interface_1.MessageCategory.OUT,
            message: {
                messageId: publishOptions.relatedMessageId || this.generateMessageId(),
                workerId: this.getWorkerId(),
                type: this.isMain() ? interface_1.MessageType.Request : interface_1.MessageType.Response,
                body: data,
                messageOptions: publishOptions,
            },
        });
    }
    publishAsync(data, publishOptions = {}) {
        return new Promise((resolve, reject) => {
            const messageId = publishOptions.relatedMessageId || this.generateMessageId();
            const handler = setTimeout(() => {
                clearTimeout(handler);
                this.asyncMessageMap.delete(messageId);
                reject(new error_1.EventBusPublishTimeoutError(messageId));
            }, publishOptions.timeout || 5000);
            this.asyncMessageMap.set(messageId, data => {
                clearTimeout(handler);
                resolve(data);
            });
            this.transit({
                messageCategory: interface_1.MessageCategory.OUT,
                message: {
                    messageId,
                    workerId: this.getWorkerId(),
                    type: this.isMain() ? interface_1.MessageType.Invoke : interface_1.MessageType.Response,
                    body: data,
                },
            });
        });
    }
    broadcast(data, options = {}) {
        if (this.isWorker()) {
            options = {
                includeMainFromWorker: false,
                includeSelfFromWorker: false,
                ...options,
            };
        }
        this.transit({
            messageCategory: interface_1.MessageCategory.OUT,
            message: {
                messageId: options.relatedMessageId || this.generateMessageId(),
                workerId: this.getWorkerId(),
                type: interface_1.MessageType.Broadcast,
                body: data,
                messageOptions: options,
            },
        });
    }
    postMessage(message) {
        var _a, _b, _c, _d, _e, _f;
        if (this.isMain()) {
            if (this.workers.length > 0) {
                if (message.type === interface_1.MessageType.Broadcast) {
                    if (message.messageOptions &&
                        message.messageOptions['relatedWorkerId']) {
                        this.workers.forEach(w => {
                            if (message.messageOptions['includeSelfFromWorker'] === false &&
                                this.getWorkerId(w) ===
                                    message.messageOptions['relatedWorkerId']) {
                                return;
                            }
                            else {
                                this.mainSendMessage(w, message);
                            }
                        });
                    }
                    else {
                        this.workers.forEach(w => this.mainSendMessage(w, message));
                    }
                }
                else if ((_a = message.messageOptions) === null || _a === void 0 ? void 0 : _a['targetWorkerId']) {
                    const targetWorker = (_c = this.workerReady.get((_b = message.messageOptions) === null || _b === void 0 ? void 0 : _b['targetWorkerId'])) === null || _c === void 0 ? void 0 : _c.worker;
                    if (!targetWorker) {
                        throw new error_1.EventBusPublishSpecifyWorkerError((_d = message.messageOptions) === null || _d === void 0 ? void 0 : _d['targetWorkerId']);
                    }
                    this.mainSendMessage(targetWorker, message);
                }
                else {
                    // round ring
                    const [worker, ...otherWorkers] = this.workers;
                    try {
                        this.mainSendMessage(worker, message);
                    }
                    catch (err) {
                        (_e = this.eventListenerMap.get(interface_1.ListenerType.Error)) === null || _e === void 0 ? void 0 : _e(new error_1.EventBusMainPostError(message, err));
                    }
                    this.workers = [...otherWorkers, worker];
                }
            }
        }
        else {
            try {
                this.workerSendMessage(message);
            }
            catch (err) {
                (_f = this.eventListenerMap.get(interface_1.ListenerType.Error)) === null || _f === void 0 ? void 0 : _f(new error_1.EventBusWorkerPostError(message, err));
            }
        }
    }
    onInited(listener) {
        this.eventListenerMap.set(interface_1.ListenerType.Inited, listener);
    }
    onPublish(listener) {
        this.eventListenerMap.set(interface_1.ListenerType.Request, listener);
    }
    onSubscribe(listener) {
        this.eventListenerMap.set(interface_1.ListenerType.Subscribe, listener);
    }
    onError(listener) {
        this.eventListenerMap.set(interface_1.ListenerType.Error, listener);
    }
    async stop() {
        this.stopping = true;
        this.hub.removeAllListeners();
        this.workerReady.clear();
        this.eventListenerMap.clear();
        this.listener = null;
        this.workers.length = 0;
    }
    generateMessageId() {
        return Math.random().toString(36).substring(2);
    }
}
exports.AbstractEventBus = AbstractEventBus;
//# sourceMappingURL=base.js.map